'use strict';

var _createClass = require('babel-runtime/helpers/create-class')['default'];

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

var _Object$keys = require('babel-runtime/core-js/object/keys')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

/**
 * `TransferableRegistry` provides ES2015 Map-like abstraction which can be
 * transfered across different scopes and realms.
 */

var TransferableRegistry = (function () {
  _createClass(TransferableRegistry, null, [{
    key: 'installed',

    /**
     * Check if registry is installed in the `scope`.
     */
    value: function installed() {
      var scope = arguments.length <= 0 || arguments[0] === undefined ? window : arguments[0];

      return scope[this.key] !== undefined;
    }

    /**
     * Resolve installed registry in the `scope`.
     *
     * This fails if no registry is installed in the `scope`.
     */
  }, {
    key: 'resolve',
    value: function resolve() {
      var scope = arguments.length <= 0 || arguments[0] === undefined ? window : arguments[0];

      (0, _invariant2['default'])(scope[this.key] !== undefined, 'Cannot resolve a registry with key "%s", it is undefined', this.key);
      var registry = scope[this.key];
      // This is a cross realm check...
      if (registry.constructor && registry.constructor.name !== this.name) {
        registry = new this(registry);
      }
      return registry;
    }
  }, {
    key: 'key',

    /**
     * Unique global key for a transferable registry instance.
     *
     * This should be set by subclasses.
     */
    value: null,
    enumerable: true
  }]);

  function TransferableRegistry() {
    var storage = arguments.length <= 0 || arguments[0] === undefined ? null : arguments[0];

    _classCallCheck(this, TransferableRegistry);

    this._storage = storage || {};
    this.initialized = storage !== null;
  }

  /**
    * React component to render a registry instance into a `<script />` element
    * from which the registry could be bootstrapped again.
    */

  /**
   * Install registry instance into the `scope` so that it could be resolved
   * later.
   */

  _createClass(TransferableRegistry, [{
    key: 'install',
    value: function install() {
      var scope = arguments.length <= 0 || arguments[0] === undefined ? window : arguments[0];

      (0, _invariant2['default'])(scope[this.constructor.key] === undefined, 'Cannot install a registry as key "%s" is already allocated in the scope', this.constructor.key);
      scope[this.constructor.key] = this._storage;
    }

    /**
     * Size of the registry.
     *
     * See ES2015 Map interface for details.
     */
  }, {
    key: 'clear',

    /**
     * Clear registry.
     *
     * See ES2015 Map interface for details.
     */
    value: function clear() {
      this._storage = {};
    }

    /**
     * Delete key from the registry.
     *
     * See ES2015 Map interface for details.
     */
  }, {
    key: 'delete',
    value: function _delete(key) {
      delete this._storage[key];
    }

    /**
     * Returns an iterator of k-v pairs.
     *
     * See ES2015 Map interface for details.
     */
  }, {
    key: 'entries',
    value: function entries() {
      var _this = this;

      return _Object$keys(this._storage).map(function (key) {
        return [key, _this._storage[key]];
      }).values();
    }

    /**
     * Returns an iterator of keys.
     *
     * See ES2015 Map interface for details.
     */
  }, {
    key: 'keys',
    value: function keys() {
      return _Object$keys(this._storage).values();
    }

    /**
     * Returns an iterator of values.
     *
     * See ES2015 Map interface for details.
     */
  }, {
    key: 'values',
    value: function values() {
      var _this2 = this;

      return _Object$keys(this._storage).map(function (key) {
        return _this2._storage[key];
      }).values();
    }

    /**
     * Call a `func` for each k-v pair in registry.
     *
     * See ES2015 Map interface for details.
     */
  }, {
    key: 'forEach',
    value: function forEach(func, context) {
      var _this3 = this;

      _Object$keys(this._storage).forEach(function (key) {
        return func.call(context, _this3._storage[key], key);
      });
    }

    /**
     * Get a value by `key`.
     *
     * See ES2015 Map interface for details.
     */
  }, {
    key: 'get',
    value: function get(key) {
      return this._storage[key];
    }

    /**
     * Check if registry has a value for `key`.
     *
     * See ES2015 Map interface for details.
     */
  }, {
    key: 'has',
    value: function has(key) {
      return key in this._storage;
    }

    /**
     * Check if registry has a value for `key`.
     *
     * See ES2015 Map interface for details.
     */
  }, {
    key: 'set',
    value: function set(key, value) {
      this._storage[key] = value;
    }

    /**
     * Get JSON serializable representation of registry.
     */
  }, {
    key: 'toJSON',
    value: function toJSON() {
      return this._storage;
    }
  }, {
    key: 'size',
    get: function get() {
      return _Object$keys(this._storage).length;
    }
  }]);

  return TransferableRegistry;
})();

exports['default'] = TransferableRegistry;
function RenderRegistry(key, _ref) {
  var registry = _ref.registry;

  if (!registry) {
    return _react2['default'].createElement('noscript', null);
  } else {
    var __html = 'window[' + JSON.stringify(key) + '] = ' + JSON.stringify(registry) + ';';
    return _react2['default'].createElement('script', { dangerouslySetInnerHTML: { __html: __html } });
  }
}

Object.defineProperty(TransferableRegistry, 'Render', {
  configurable: true,
  get: function get() {
    var value = RenderRegistry.bind(null, this.key);
    Object.defineProperty(this, 'Render', { value: value });
    return value;
  }
});
module.exports = exports['default'];
