'use strict';

var _extends = require('babel-runtime/helpers/extends')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports['default'] = createPage;

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _utilError = require('./util/Error');

function createPage(spec, key) {
  (0, _invariant2['default'])(key !== undefined, 'oops');
  var route = spec.route;
  var meta = spec.meta;
  var path = spec.path;
  var component = spec.component;
  var indexRoute = spec.indexRoute;
  var _spec$childRoutes = spec.childRoutes;
  var childRoutes = _spec$childRoutes === undefined ? [] : _spec$childRoutes;

  if (route) {
    if (isContextModule(route)) {
      return _extends({}, createContextRoute(path, route, component), { key: key });
    } else {
      return _extends({}, createProxyRoute(path, route, component), { key: key });
    }
  }

  return {
    type: 'Module',
    meta: meta,
    key: key,
    path: path,
    component: component,
    childRoutes: childRoutes,
    indexRoute: indexRoute
  };
}

function createContextRoute(path, context, component) {
  var childRoutes = [];
  var indexRoute = undefined;
  context.keys().forEach(function (key) {
    var module = function module(cb) {
      return ensureModuleLoaded(context(key), cb);
    };
    var childPath = key.substring(1).replace(/\..+$/, '').replace(/^\//, '');
    if (childPath === 'index') {
      indexRoute = createProxyRoute(path, module);
    } else {
      childRoutes.push(createProxyRoute(childPath, module));
    }
  });
  childRoutes.sort(sortChildRoutes);
  return {
    type: 'Context',
    path: path,
    childRoutes: childRoutes,
    indexRoute: indexRoute,
    component: component,

    getMeta: function getMeta(callback) {
      if (indexRoute) {
        indexRoute.getMeta(callback);
      } else {
        callback(null, null);
      }
    },

    getKey: function getKey(callback) {
      if (indexRoute) {
        indexRoute.getKey(callback);
      } else {
        callback(null, null);
      }
    }
  };
}

function ensureModuleLoaded(module, callback) {
  if (isModuleLoader(module)) {
    module(callback);
  } else {
    callback(module);
  }
}

function createProxyRoute(path, module) {
  function cache(module) {
    self.meta = module.meta;
    self.key = module.key;
  }
  var self = {
    type: 'Proxy',

    path: path,

    getKey: function getKey(callback) {
      ensureModuleLoaded(module, function (module) {
        cache(module);
        if (module.key) {
          callback(null, module.key);
        } else if (module.getKey) {
          module.getKey(function (error, key) {
            if (error) {
              callback(error);
            } else {
              callback(null, key);
            }
          });
        } else {
          (0, _invariant2['default'])(false, 'Cannot get key of the module');
        }
      });
    },

    getMeta: function getMeta(callback) {
      ensureModuleLoaded(module, function (module) {
        cache(module);
        if (module.meta) {
          callback(null, module.meta);
        } else if (module.getMeta) {
          module.getMeta(function (error, meta) {
            if (error) {
              callback(error);
            } else {
              callback(null, meta);
            }
          });
        } else {
          callback(null, {});
        }
      });
    },

    getIndexRoute: function getIndexRoute(location, callback) {
      ensureModuleLoaded(module, function (module) {
        cache(module);
        if (module.type === 'Proxy') {
          module.getIndexRoute(location, callback);
        } else if (module.indexRoute !== undefined) {
          callback(null, module.indexRoute);
        } else if (module.getIndexRoute) {
          (0, _utilError.suggestPullRequest)('Route.getIndexRoute');
        } else {
          callback(null, undefined);
        }
      });
    },

    getComponent: function getComponent(location, callback) {
      ensureModuleLoaded(module, function (module) {
        cache(module);
        if (module.type === 'Proxy') {
          module.getComponent(location, callback);
        } else if (module.component !== undefined) {
          callback(null, module.component);
        } else if (module.getComponent) {
          (0, _utilError.suggestPullRequest)('Route.getChildRoutes');
        } else {
          callback(null, undefined);
        }
      });
    },

    getChildRoutes: function getChildRoutes(location, callback) {
      ensureModuleLoaded(module, function (module) {
        cache(module);
        if (module.type === 'Proxy') {
          module.getChildRoutes(location, callback);
        } else if (module.childRoutes !== undefined) {
          callback(null, module.childRoutes);
        } else if (module.getChildRoutes) {
          (0, _utilError.suggestPullRequest)('Route.getChildRoutes');
        } else {
          callback(null, []);
        }
      });
    }
  };

  return self;
}

/**
 * Check if `module` is a context module.
 */
function isContextModule(module) {
  return typeof module === 'function' && typeof module.keys === 'function' && typeof module.resolve === 'function';
}

/**
 * Check if `module` is a module loader (created with bundle-loader).
 */
function isModuleLoader(module) {
  return typeof module === 'function' && !isContextModule(module);
}

/**
 * Sort function for child routes.
 */
function sortChildRoutes(a, b) {
  if (a.path.length > b.path.length) {
    return 1;
  } else if (a.path.length > b.path.length) {
    return -1;
  } else {
    return 0;
  }
}
module.exports = exports['default'];
