'use strict';

var _extends = require('babel-runtime/helpers/extends')['default'];

var _regeneratorRuntime = require('babel-runtime/regenerator')['default'];

var _Promise = require('babel-runtime/core-js/promise')['default'];

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports.pathConcat = pathConcat;
exports.getIndexRoute = getIndexRoute;
exports.getChildRoutes = getChildRoutes;
exports.collectRoutes = collectRoutes;

var _Promise2 = require('./Promise');

function flattenArray(array) {
  var result = [];
  for (var i = 0; i < array.length; i++) {
    result = result.concat(array[i]);
  }
  return result;
}

function loadRouteKey(route) {
  var key;
  return _regeneratorRuntime.async(function loadRouteKey$(context$1$0) {
    while (1) switch (context$1$0.prev = context$1$0.next) {
      case 0:
        if (!route.getKey) {
          context$1$0.next = 7;
          break;
        }

        context$1$0.next = 3;
        return _regeneratorRuntime.awrap((0, _Promise2.awaitCallback)(route.getKey));

      case 3:
        key = context$1$0.sent;
        return context$1$0.abrupt('return', _extends({}, route, { key: key }));

      case 7:
        return context$1$0.abrupt('return', route);

      case 8:
      case 'end':
        return context$1$0.stop();
    }
  }, null, this);
}

function loadRouteMeta(route) {
  var meta;
  return _regeneratorRuntime.async(function loadRouteMeta$(context$1$0) {
    while (1) switch (context$1$0.prev = context$1$0.next) {
      case 0:
        if (!route.getMeta) {
          context$1$0.next = 7;
          break;
        }

        context$1$0.next = 3;
        return _regeneratorRuntime.awrap((0, _Promise2.awaitCallback)(route.getMeta));

      case 3:
        meta = context$1$0.sent;
        return context$1$0.abrupt('return', _extends({}, route, { meta: meta }));

      case 7:
        return context$1$0.abrupt('return', route);

      case 8:
      case 'end':
        return context$1$0.stop();
    }
  }, null, this);
}

function pathConcat(a, b) {
  if (!a) {
    return b;
  }
  if (!b) {
    return a;
  }
  return (a + '/' + b).replace(/\/+/g, '/');
}

function getIndexRoute(route) {
  return _regeneratorRuntime.async(function getIndexRoute$(context$1$0) {
    while (1) switch (context$1$0.prev = context$1$0.next) {
      case 0:
        if (!route.indexRoute) {
          context$1$0.next = 4;
          break;
        }

        return context$1$0.abrupt('return', route.indexRoute);

      case 4:
        if (!route.getIndexRoute) {
          context$1$0.next = 8;
          break;
        }

        return context$1$0.abrupt('return', (0, _Promise2.awaitCallback)(route.getIndexRoute, null));

      case 8:
        return context$1$0.abrupt('return', undefined);

      case 9:
      case 'end':
        return context$1$0.stop();
    }
  }, null, this);
}

function getChildRoutes(route) {
  return _regeneratorRuntime.async(function getChildRoutes$(context$1$0) {
    while (1) switch (context$1$0.prev = context$1$0.next) {
      case 0:
        if (!route.childRoutes) {
          context$1$0.next = 4;
          break;
        }

        return context$1$0.abrupt('return', route.childRoutes);

      case 4:
        if (!route.getChildRoutes) {
          context$1$0.next = 8;
          break;
        }

        return context$1$0.abrupt('return', (0, _Promise2.awaitCallback)(route.getChildRoutes, null));

      case 8:
        return context$1$0.abrupt('return', []);

      case 9:
      case 'end':
        return context$1$0.stop();
    }
  }, null, this);
}

function collectRoutes(route) {
  var routes, indexRoute, childRoutes;
  return _regeneratorRuntime.async(function collectRoutes$(context$1$0) {
    while (1) switch (context$1$0.prev = context$1$0.next) {
      case 0:
        routes = [route];
        context$1$0.next = 3;
        return _regeneratorRuntime.awrap(getIndexRoute(route));

      case 3:
        indexRoute = context$1$0.sent;

        if (indexRoute) {
          indexRoute = _extends({}, indexRoute, { path: route.path });
          routes.push(indexRoute);
        }

        context$1$0.next = 7;
        return _regeneratorRuntime.awrap(getChildRoutes(route));

      case 7:
        childRoutes = context$1$0.sent;

        if (!(childRoutes.length > 0)) {
          context$1$0.next = 15;
          break;
        }

        context$1$0.next = 11;
        return _regeneratorRuntime.awrap(_Promise.all(childRoutes.map(collectRoutes)));

      case 11:
        childRoutes = context$1$0.sent;

        childRoutes = flattenArray(childRoutes);
        childRoutes = childRoutes.map(function (childRoute) {
          return _extends({}, childRoute, {
            path: pathConcat(route.path, childRoute.path)
          });
        });
        routes = routes.concat(childRoutes);

      case 15:
        context$1$0.next = 17;
        return _regeneratorRuntime.awrap(_Promise.all(routes.map(loadRouteKey)));

      case 17:
        routes = context$1$0.sent;
        context$1$0.next = 20;
        return _regeneratorRuntime.awrap(_Promise.all(routes.map(loadRouteMeta)));

      case 20:
        routes = context$1$0.sent;
        return context$1$0.abrupt('return', routes);

      case 22:
      case 'end':
        return context$1$0.stop();
    }
  }, null, this);
}
