'use strict';

var _defineProperty = require('babel-runtime/helpers/define-property')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports['default'] = configure;

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

var _extractTextWebpackPlugin = require('extract-text-webpack-plugin');

var _extractTextWebpackPlugin2 = _interopRequireDefault(_extractTextWebpackPlugin);

var _RenderStaticPlugin = require('./RenderStaticPlugin');

var _RenderStaticPlugin2 = _interopRequireDefault(_RenderStaticPlugin);

var _LogProgressPlugin = require('./LogProgressPlugin');

var _LogProgressPlugin2 = _interopRequireDefault(_LogProgressPlugin);

var _babelQueryAPIBabelPlugin = require('../babel/QueryAPIBabelPlugin');

var _babelQueryAPIBabelPlugin2 = _interopRequireDefault(_babelQueryAPIBabelPlugin);

var JS_BUNDLE_NAME = 'bootstrap.js';
exports.JS_BUNDLE_NAME = JS_BUNDLE_NAME;
var CSS_BUNDLE_NAME = 'bootstrap.css';

exports.CSS_BUNDLE_NAME = CSS_BUNDLE_NAME;
var babelLoader = require.resolve('babel-loader');
var cssLoader = require.resolve('css-loader');
var cssLoaderLocals = require.resolve('css-loader/locals');
var cssModuleLoader = cssLoader + '?modules';
var cssModuleLoaderLocals = cssLoaderLocals + '?modules';
var styleLoader = require.resolve('style-loader');
var fileLoader = require.resolve('file-loader');
var jsonLoader = require.resolve('json-loader');
var urlLoader = require.resolve('url-loader');
var sitegenLoaderMarkdown = require.resolve('sitegen-loader-markdown');
var webpackHotMiddlewareClient = require.resolve('webpack-hot-middleware/client');
var babelPluginReactTransform = require.resolve('babel-plugin-react-transform');
var reactTransformHMR = require.resolve('react-transform-hmr');

function configure() {
  var _alias, _alias2;

  var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

  var build = options.mode === 'build' && !options.dev;
  var serve = options.mode === 'serve' && !options.dev;
  var serveDev = options.mode === 'serve' && options.dev;

  var plugins = [new _LogProgressPlugin2['default'](options.compilerName), build && new _RenderStaticPlugin2['default'](), serveDev && new _webpack2['default'].optimize.OccurenceOrderPlugin(), serveDev && new _webpack2['default'].HotModuleReplacementPlugin(), serveDev && new _webpack2['default'].NoErrorsPlugin(), serve && new _webpack2['default'].optimize.UglifyJsPlugin(), (build || serve) && new _extractTextWebpackPlugin2['default'](CSS_BUNDLE_NAME, { allChunks: true })];

  var entry = [serveDev && webpackHotMiddlewareClient + '?reload=true'].concat(options.entry);

  return {
    entry: entry.filter(Boolean),
    target: build ? 'node' : 'web',
    sitegen: options,
    devtool: serveDev ? 'cheap-module-eval-source-map' : undefined,
    babel: {
      plugins: [_babelQueryAPIBabelPlugin2['default'], serveDev ? babelPluginReactTransform : null].filter(Boolean),
      extra: {
        'react-transform': {
          transforms: [{
            transform: reactTransformHMR,
            imports: ['react'],
            locals: ['module']
          }]
        }
      }
    },
    stats: {
      children: false,
      assets: false,
      hash: false,
      version: false,
      timings: false
    },
    output: {
      publicPath: options.publicPath || '/',
      library: 'SitegenSite',
      libraryTarget: build ? 'commonjs2' : 'var',
      path: options.output,
      filename: JS_BUNDLE_NAME,
      chunkFilename: '[name].js'
    },
    plugins: plugins.filter(Boolean),
    resolve: {
      alias: (_alias = {}, _defineProperty(_alias, 'sitegen/internal', require.resolve('../internal')), _defineProperty(_alias, 'sitegen', require.resolve('../')), _defineProperty(_alias, 'site', options.lib), _alias)
    },
    resolveLoader: {
      alias: (_alias2 = {}, _defineProperty(_alias2, 'page', require.resolve('./loader/page')), _defineProperty(_alias2, 'page-id', require.resolve('./loader/page-id')), _defineProperty(_alias2, 'site', require.resolve('./loader/site')), _alias2)
    },
    module: {
      loaders: [
      // content
      {
        test: /\.json$/,
        loader: jsonLoader
      }, {
        test: /\.js$/,
        loader: babelLoader + '?stage=0',
        exclude: /node_modules/
      }, {
        test: /\.md$/,
        loader: babelLoader + '!' + sitegenLoaderMarkdown
      },

      // styles
      {
        test: /\.css$/,
        exclude: /\.module\.css$/,
        loader: serve ? // eslint-disable-line no-nested-ternary
        _extractTextWebpackPlugin2['default'].extract(styleLoader, cssLoader) : serveDev ? styleLoader + '!' + cssLoader : cssLoaderLocals
      }, {
        test: /\.module\.css$/,
        loader: serve ? // eslint-disable-line no-nested-ternary
        _extractTextWebpackPlugin2['default'].extract(styleLoader, cssModuleLoader) : serveDev ? styleLoader + '!' + cssModuleLoader : cssModuleLoaderLocals
      },

      // images
      {
        test: /\.png$/,
        loader: urlLoader + '?prefix=img/&limit=5000'
      }, {
        test: /\.jpg$/,
        loader: urlLoader + '?prefix=img/&limit=5000'
      }, {
        test: /\.gif$/,
        loader: urlLoader + '?prefix=img/&limit=5000'
      },

      // fonts
      {
        test: /\.eot(\?[a-z0-9]+)?$/,
        loader: fileLoader + '?prefix=font/'
      }, {
        test: /\.ttf(\?[a-z0-9]+)?$/,
        loader: fileLoader + '?prefix=font/'
      }, {
        test: /\.svg(\?[a-z0-9]+)?$/,
        loader: fileLoader + '?prefix=font/'
      }, {
        test: /\.woff(\?[a-z0-9]+)?$/,
        loader: urlLoader + '?prefix=img/&limit=5000'
      }, {
        test: /\.woff2(\?[a-z0-9]+)?$/,
        loader: urlLoader + '?prefix=img/&limit=5000'
      }]
    }
  };
}
